@@ -15,20 +15,21 @@
  *See the GNU General Public License for more details.
  *
  *You should have received a copy of the
- *GNU General Public License along with Goupil;
+ *GNU General Public License along with Nemiver;
  *see the file COPYING.
  *If not, write to the Free Software Foundation,
  *Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *See COPYRIGHT file copyright information.
  */
-#include "nmv-safe-ptr-utils.h"
+#include "common/nmv-safe-ptr-utils.h"
+#include "common/nmv-connection.h"
+#include "common/nmv-connection-manager.h"
+#include "common/nmv-tools.h"
+#include "common/nmv-transaction.h"
+#include "common/nmv-sql-statement.h"
+#include "nmv-ui-utils.h"
 #include "nmv-sess-mgr.h"
-#include "nmv-connection.h"
-#include "nmv-connection-manager.h"
-#include "nmv-tools.h"
-#include "nmv-transaction.h"
-#include "nmv-sql-statement.h"
 
 using namespace std ;
 using namespace nemiver::common ;
@@ -38,9 +39,9 @@ using nemiver::common::ConnectionManager ;
 using nemiver::common::Transaction ;
 using nemiver::common::SQLStatement ;
 
-static const char *REQUIRED_DB_SCHEMA_VERSION = "1.0" ;
+static const char *REQUIRED_DB_SCHEMA_VERSION = "1.2" ;
 
-namespace nemiver {
+NEMIVER_BEGIN_NAMESPACE (nemiver)
 
 class SessMgr : public ISessMgr {
     //non copyable
@@ -94,27 +95,54 @@ struct SessMgr::Priv {
         return conn;
     }
 
-    UString path_to_create_table_script ()
+    UString path_to_create_tables_script ()
     {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
         string path = Glib::build_filename
                                     (Glib::locale_from_utf8 (root_dir),
                                      "sqlscripts/create-tables.sql") ;
         return Glib::locale_to_utf8 (path) ;
     }
 
+    UString path_to_drop_tables_script ()
+    {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+        string path = Glib::build_filename
+                                    (Glib::locale_from_utf8 (root_dir),
+                                     "sqlscripts/drop-tables.sql") ;
+        return Glib::locale_to_utf8 (path) ;
+    }
+
     bool create_db ()
     {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
 
-        UString path_to_script = path_to_create_table_script () ;
+        UString path_to_script = path_to_create_tables_script () ;
         Transaction transaction (*conn) ;
         return tools::execute_sql_command_file
-                                        (path_to_create_table_script (),
+                                        (path_to_script,
+                                         transaction,
+                                         cerr) ;
+    }
+
+    bool drop_db ()
+    {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+        UString path_to_script = path_to_drop_tables_script () ;
+        Transaction transaction (*conn) ;
+        return tools::execute_sql_command_file
+                                        (path_to_script,
                                          transaction,
                                          cerr) ;
     }
 
     bool check_db_version ()
     {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
+        NEMIVER_TRY
+
         SQLStatement query ("select version from schemainfo") ;
 
         RETURN_VAL_IF_FAIL (connection ()->execute_statement (query), false) ;
@@ -122,42 +150,53 @@ struct SessMgr::Priv {
         UString version ;
         RETURN_VAL_IF_FAIL (connection ()->get_column_content (0, version),
                             false) ;
-        LOG_D ("version: " << version, NMV_DEFAULT_DOMAIN) ;
+        LOG_DD ("version: " << version) ;
         if (version != REQUIRED_DB_SCHEMA_VERSION) {
             return false ;
         }
+        NEMIVER_CATCH_AND_RETURN (false)
         return true ;
     }
 
     void init_db ()
     {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+        NEMIVER_TRY
         //if the db version is not what we expect, create
         //a new db with the schema we expect.
         if (!check_db_version ()) {
+            drop_db () ;
             THROW_IF_FAIL (create_db ()) ;
         }
+        NEMIVER_CATCH
     }
 
     void init ()
     {
+        LOG_FUNCTION_SCOPE_NORMAL_DD ;
+
+        NEMIVER_TRY
         init_db () ;
+        NEMIVER_CATCH
     }
 };//end struct SessMgr::Priv
 
 SessMgr::SessMgr ()
 {
-    m_priv = new SessMgr::Priv ;
+    m_priv.reset (new SessMgr::Priv) ;
     m_priv->init () ;
 }
 
 SessMgr::SessMgr (const UString &a_root_dir)
 {
-    m_priv = new SessMgr::Priv (a_root_dir) ;
+    m_priv.reset (new SessMgr::Priv (a_root_dir)) ;
     m_priv->init () ;
 }
 
 SessMgr::~SessMgr ()
 {
+    LOG_D ("delete", "destructor-domain") ;
 }
 
 Transaction&
@@ -166,7 +205,8 @@ SessMgr::default_transaction ()
     THROW_IF_FAIL (m_priv) ;
 
     if (!m_priv->default_transaction) {
-        m_priv->default_transaction = new Transaction (*m_priv->connection ()) ;
+        m_priv->default_transaction =
+            TransactionSafePtr (new Transaction (*m_priv->connection ())) ;
         THROW_IF_FAIL (m_priv->default_transaction) ;
     }
     return *m_priv->default_transaction ;
@@ -261,7 +301,8 @@ SessMgr::store_session (Session &a_session,
                 + UString::from_int (a_session.session_id ()) + ", '"
                 + break_iter->file_name () + "', '"
                 + break_iter->file_full_name () + "', "
-                + UString::from_int (break_iter->line_number ())
+                + UString::from_int (break_iter->line_number ()) + ", "
+                + UString::from_int (break_iter->enabled ())
                 + ")"
                 ;
         THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
@@ -282,6 +323,22 @@ SessMgr::store_session (Session &a_session,
                 + "')" ;
         THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
     }
+
+    //store the search paths
+    query = "delete from searchpaths where sessionid = "
+            + UString::from_int (a_session.session_id ()) ;
+    THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
+
+    list<UString>::const_iterator path_iter ;
+    for (path_iter = a_session.search_paths ().begin ();
+         path_iter != a_session.search_paths ().end ();
+         ++path_iter) {
+        query = "insert into searchpaths values(NULL, "
+                + UString::from_int (a_session.session_id ()) + ", '"
+                + *path_iter
+                + "')" ;
+        THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
+    }
     trans.end () ;
 }
 
@@ -340,22 +397,38 @@ SessMgr::load_session (Session &a_session,
 
     //load the breakpoints
     query = "select breakpoints.filename, breakpoints.filefullname, "
-            "breakpoints.linenumber from "
+            "breakpoints.linenumber, breakpoints.enabled from "
             "breakpoints where breakpoints.sessionid = "
             + UString::from_int (session.session_id ())
             ;
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
     while (trans.get ().get_connection ().read_next_row ()) {
-        UString filename, filefullname, linenumber;
+        UString filename, filefullname, linenumber, enabled;
         THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
                                                                 (0, filename));
         THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
                                                             (1, filefullname));
         THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
                                                                 (2, linenumber));
+        THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                                (3, enabled));
         session.breakpoints ().push_back (SessMgr::BreakPoint (filename,
                                                                filefullname,
-                                                               linenumber)) ;
+                                                               linenumber,
+                                                               enabled)) ;
+    }
+
+    //load the search paths
+    query = "select searchpaths.path from "
+            "searchpaths where searchpaths.sessionid = "
+            + UString::from_int (session.session_id ())
+            ;
+    THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
+    while (trans.get ().get_connection ().read_next_row ()) {
+        UString path;
+        THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                                (0, path));
+        session.search_paths ().push_back (path) ;
     }
 
     //load the opened files
@@ -465,6 +538,10 @@ SessMgr::clear_session (gint64 a_id, Transaction &a_trans)
             "sessionid = " + UString::from_int (a_id) ;
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
 
+    query = "delete from searchpaths where "
+            "sessionid = " + UString::from_int (a_id) ;
+    THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query)) ;
+
     trans.end () ;
 }
 
@@ -480,6 +557,6 @@ ISessMgr::create (const UString &a_root_path)
     return ISessMgrSafePtr (new SessMgr (a_root_path)) ;
 }
 
-}//end namespace nemiver
+NEMIVER_END_NAMESPACE (nemiver)
 
 
